<?php

namespace Service\Integration;

use Doctrine\ORM\EntityManager;
use Exception\CustomException;
use Util\InstanceProvider;
use Util\Messages;
use Zeedhi\DevOps\Model\Configurationmanager\Entities\Deploy;
use Zeedhi\DevOps\Model\Configurationmanager\Entities\ProductFiles;
use Zeedhi\Framework\DTO\Request;
use Util\Utilities;

class DEVOPSIntegration {

    const CLIENT_ID = "cd7e82153be60953002055f9b19a50a4";
    const CLIENT_SECRET = "do25449996c21f780fe30badd6521931";

    const DEPLOY_EVENT_TYPE = "D";

    protected $entityManager;
    protected $devOpsURL;

    /** @var \Zeedhi\DevOps\Model\Devops\Repositories\ProductVersionPackages $productVersionPackagesRepository */
    protected $productVersionPackagesRepository;

    /**
     * @param EntityManager $entityManager
     * @param Utilities $environment
     * @param $devOpsURL
     */
    public function __construct(EntityManager $entityManager, Utilities $environment, $devOpsURL){
        $this->entityManager = $entityManager;
        $this->environment   = $environment;
        $this->devOpsURL     = $devOpsURL;

        $this->productVersionPackagesRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Devops\Entities\ProductVersionPackages');
    }

    /**
     * @param $productId
     * @param $productServerId
     * @param $productPath
     * @param $isOnlyOnePackage
     * @param array $product
     * @return mixed|null
     * @throws \Exception
     */
    public function createDeploy($productId, $productServerId, array $product, $productPath, $isOnlyOnePackage = false) {
        $productVersionId    = $product['PRODUCT_VERSION_ID'];
        /** @var \Zeedhi\DevOps\Model\Devops\Entities\ProductVersionPackages[] $prodVersionPackages */
        $prodVersionPackages = $this->productVersionPackagesRepository->findBy(array('productVersion' => $productVersionId));
        $prodPackages = array();

        foreach ($prodVersionPackages as $prodVersion) {
            if($prodVersion->getPathToDeploy() == $productPath) {
                $prodPackages[] = $prodVersion->getId();
            }
        }

        if(count($prodPackages) > 1 && $isOnlyOnePackage) {
            throw new CustomException('ERR_VERSION_PATH_TO_DEPLOY_DUPLICATED');
        }

        $deployRow = array(array(
            'ID'                 => null,
            'IS_ACTIVE'          => Deploy::IS_ACTIVE_YES,
            'STATUS'             => Deploy::PENDING_DEPLOY_STATUS,
            'NAME'               => $product['NAME'],
            'PACKAGE'            => $prodPackages,
            'PRODUCT_ID'         => $productId,
            'PRODUCT_VERSION_ID' => $productVersionId,
            'SERVER'             => [$productServerId],
            'DEVOPS_USER_ID'     => InstanceProvider::getDevOpsEnvironment()->getCurrentUserId()
        ));

        $dataSet = $this->getDataRequestDataSet($deployRow);
        return $this->requestDevOpsRoute('/deploy/save', $dataSet);
    }

    /**
     * @param $productId
     * @param $productServerId
     * @param $productVersionId
     * @param $serverIsWindows
     * @return mixed|null
     * @throws \Exception
     */
    public function getProductDeployScripts($productId, $productServerId, $productVersionId, $serverIsWindows) {
        $filter = array (
            array (
                "name"     => "PRODUCT_ID",
                "operator" => "=",
                "value"    => $productId
            ), array (
                "name"     => "TYPE",
                "operator" => "IN",
                "value"    => array (
                    ProductFiles::DEPLOY_SCRIPT_BEFORE,
                    ProductFiles::DEPLOY_SCRIPT_AFTER,
                    ProductFiles::BACKUP_SCRIPT_BEFORE,
                    ProductFiles::BACKUP_SCRIPT_AFTER,
                    ProductFiles::OTHERS_FILES
                )
            ), array (
                "name"     => "PRODUCT_SERVER_ID",
                "operator" => "=",
                "value"    => $productServerId
            ), array (
                "name"     => "PRODUCT_VERSION_ID",
                "operator" => "=",
                "value"    => $productVersionId
            ), array (
                "name"     => "TYPE_EVENT",
                "operator" => "=",
                "value"    => self::DEPLOY_EVENT_TYPE
            ), array (
                "name"     => "IS_WINDOWS",
                "operator" => "=",
                "value"    => $serverIsWindows
            )
        );

        $filterRequestData = $this->getDataRequestFilter($filter);
        return $this->requestDevOpsRoute('/findVersionProductFiles', $filterRequestData);
    }

    /**
     * @return mixed|null
     * @throws \Exception
     */
    public function getDevOpsToken() {
        if($this->environment->getCurrentDevOpsToken()) {
            $token = $this->environment->getCurrentDevOpsToken();
        } else {
            $token = $this->requestToken();
            $this->environment->setCurrentDevOpsToken($token);
        }

        return $token;
    }

    public function findProductVersion($productDevOpsId,$pageSize,$page,$filter = array()) {
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $filterString = \Zeedhi\DevOps\Util\Utilities::createQueryFiltered($filter);
        $versions = InstanceProvider::getRepository()->getProductVersion()
            ->findProductVersionWithConcatVersionByProduct($productDevOpsId,$this->environment->getPmCurrentOrganizationId(),$limit,$offset,$filterString);
        return $versions;
    }

    /**
     * @return mixed
     * @throws \Exception
     */
    private function requestToken(){
        $data = array(
            "CLIENT_ID"     => self::CLIENT_ID,
            "CLIENT_SECRET" => self::CLIENT_SECRET
        );

        $data = json_encode($data);
        $url = $this->devOpsURL . '/do_requestToken';

        $header = array();
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: ' . strlen($data);
        $header[] = 'OAuth-KeepConnected: Yes';

        $curl_session = curl_init();

        curl_setopt($curl_session, CURLOPT_HTTPHEADER, $header);
        curl_setopt($curl_session, CURLOPT_URL, $url);
        curl_setopt($curl_session, CURLOPT_POSTFIELDS, $data);
        curl_setopt($curl_session, CURLOPT_RETURNTRANSFER, true);

        $result = curl_exec($curl_session);
        $result = json_decode($result, true);
        $error  = curl_error($curl_session);

        curl_close($curl_session);

        if (!empty($result['error'])) {
            throw new \Exception($result['error']);
        } else if (!empty($error)) {
            throw new \Exception($error);
        } else {
            return $result["dataset"]["token"][0];
        }
    }

    /**
     * @param $uri
     * @param array $data
     * @return mixed|null
     * @throws \Exception
     */
    public function requestDevOpsRoute($uri, $data = array()){
        $data = json_encode($data);
        $url = $this->devOpsURL . $uri;

        $header = array();
        $header[] = 'OAuth-Token: ' . $this->getDevOpsToken();
        $header[] = 'OAuth-SecretId: ' . self::CLIENT_SECRET;
        $header[] = 'OAuth-KeepConnected: Yes';
        $header[] = 'Token-login: ' . $this->environment->getToken();
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: ' . strlen($data);
        $header[] = 'Expect:';

        $curl_session = curl_init();

        curl_setopt($curl_session, CURLOPT_HTTPHEADER, $header);
        curl_setopt($curl_session, CURLOPT_URL, $url);
        curl_setopt($curl_session, CURLOPT_POSTFIELDS, $data);
        curl_setopt($curl_session, CURLOPT_TIMEOUT, 86400);
        curl_setopt($curl_session, CURLOPT_RETURNTRANSFER, true);

        $result = curl_exec($curl_session);
        $result = json_decode($result, true);
        $error  = curl_error($curl_session);

        curl_close($curl_session);

        if (!empty($result['error'])) {
            throw new \Exception($result['error']);
        } else if (!empty($error)) {
            throw new \Exception($error);
        } else if (!empty($result['messages'])) {
            throw new \Exception($result['messages'][0]['message']);
        } else {
            $result = isset($result['dataset']) ? $result['dataset'] : null;
        }

        return $result;
    }

    /**
     * @param $dataSet
     * @return array
     */
    private function getDataRequestDataSet($dataSet) {
        $dataSet = array(
            "requestType" => Request::TYPE_DATA_SET,
            "dataset" => $dataSet
        );

        return $dataSet;
    }

    /**
     * @param array $filter
     * @param int $itemsPerPage
     * @param int $page
     * @return array
     */
    private function getDataRequestFilter($filter = array(), $itemsPerPage = 500, $page = 1){
        $dataFilter = array(
            "requestType" => Request::TYPE_FILTER,
            "filter" => $filter,
            "itemsPerPage" => $itemsPerPage,
            "page" => $page
        );

        return $dataFilter;
    }
}
