var Project = function(metaDataFactory, templateManager) {

	var self = this;
	var ZEEDHI_WEB_SOCKET = "ws://ws.zeedhi.com";
	var dependenciesFolder = templateManager.dependenciesFolder || "bower_components";
	var backendDependenciesFolder = templateManager.backendDependenciesFolder || "../backend";

	var DEFAULT_VALUES = {
		usePostData: false,
		showMenu: true,
		showHeader: true,
		componentMessages: "component/messages.html",
		footer: "component/footer-edit-wizard.html",
		mainWindow: "login",
		logLevel: "DEBUG",
		aclEnable: false,
		aclDefaultRule: "I",
		shouldBackToMainWindowOnReload: false,
		inDeveloperMode: templateManager.inDeveloperMode,
		zhStoragePath : "",
		developerStudioUrl : "http://ds.zeedhi.com/",
		developerWebSocketUrl: templateManager.developerWebSocketUrl || ZEEDHI_WEB_SOCKET,
		zhFeaturesWebSocketUrl: templateManager.zhFeaturesWebSocketUrl || ZEEDHI_WEB_SOCKET,
		updateTemplateAfterWebSocketMessage: true,
		versionFile: templateManager.versionFile || "./bower.json",
		backendVersionFile: templateManager.backendVersionFile || "./../backend/composer.json",
		dependenciesFolder: dependenciesFolder,
		backendDependenciesFolder: backendDependenciesFolder,
		zhFeatureUsageEnabled: undefined, //Check MetaDataFactory.projectFactory to understand,
		timeout: 15000,
		maintainHistoryItems: false,
		filterLinkedToPreference: false,
		customFloatingActions: [],
		iconSpriteFile: dependenciesFolder + "/zeedhi-frontend/assets/images/icons/sprite-zeedhi.svg",
		showDescriptionOrder: true
	};

	self.addMenu = function(menu, parentName) {
		if (menu) {
			var parent;
			if (parentName) {
				parent = self.getMenu(parentName);
			}

			menu = metaDataFactory.menuFactory(menu, parent);

			if (parent) {
				if (Util.isArray(parent.menus)) {
					parent.menus.push(menu);
				} else {
					parent.menus = [menu];
				}
			} else {
				self.groupMenu.push(menu);
			}
			templateManager.updateTemplate();
		}
	};

	self.addMenus = function(menus) {
		if (Array.isArray(menus)) {
			menus.forEach(function(menu) {
				self.addMenu(menu, menu.parent);
			});
		}
	};

	self.removeMenu = function(name) {
		var menu = self.getMenu(name);
		if (menu) {
			if (menu.parent) {
				menu.parent.menus.splice(menu.parent.menus.indexOf(menu), 1);
			} else {
				self.groupMenu.splice(self.groupMenu.indexOf(menu), 1);
			}
			templateManager.updateTemplate();
		}
	};

	function _getMenu(groupMenu, name) {
		var menu;
		if (Util.isArray(groupMenu) && groupMenu.length > 0) {
			for (var i = 0; i < groupMenu.length && !menu; i++) {
				if (name === groupMenu[i].name) {
					menu = groupMenu[i];
				} else if (groupMenu[i].menus) {
					menu = _getMenu(groupMenu[i].menus, name);
				}
			}
		}
		return menu;
	}

	self.getMenu = function(name) {
		return _getMenu(self.groupMenu, name);
	};

	var _setDefaultValuesField = (function(project) {
		Util.extend(project, DEFAULT_VALUES);
	})(self);
};